<?php
/*
Plugin Name: Jabze Chat
Plugin URI: http://www.jabze.com/site/page/web-based-chat-for-wordpress
Description: Jabze is an industrial strength and IT friendly messenger service, with a great mobile experience. Jabze Chat is
the WordPress extension to enable chat function on a WordPress site for signed-in users. It is most useful in an intranet
environment. 
Author: jabze
Version: 1.3.4
Author URI: http://www.jabze.com
License: GPLv2 or later
*/

/* Copyright 2013 Sumilux Technologies, Inc. (email: info@sumilux.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/


define('JABZE_VERSION', '1.3.4');
define('JABZE_PLUGIN_URL', plugin_dir_url(__FILE__));
define('JABZE_APP_NAME', 'Jabze Chat');
define('JABZE_CHAT_URL', 'http://wpsvc.jabze.com/http-bind/');
//TODO use ssl 
define('JABZE_RESTFUl_URL', 'https://wpsvc.jabze.com/');
define('JABZE_SETUP_URL', JABZE_RESTFUl_URL . 'api/wordpressSetUp/version/'.JABZE_VERSION);
define('JABZE_SYNC_URL', JABZE_RESTFUl_URL . 'api/wordPressSync/version/'.JABZE_VERSION);
define('JABZE_SEND_CODE_URL', JABZE_RESTFUl_URL . 'api/wordpressSendCode/version/'.JABZE_VERSION);
define('JABZE_CHECK_STATUS_URL', JABZE_RESTFUl_URL . 'api/wordpressCheckStatus/version/'.JABZE_VERSION);
define('JABZE_CHECK_VERSION_URL', JABZE_RESTFUl_URL . 'api/wordpressCheckVersion/version/'.JABZE_VERSION);
define('JABZE_SEND_LOG_URL', JABZE_RESTFUl_URL . 'api/log');


$jabze_domain_name = 'jabze_domain';
$jabze_access_token_name = 'jabze_access_token';
$jabze_email_name='jabze_email';
$jabze_auth_secret_name='jabze_auth_secret';
$jabze_setup_error_code_name = 'jabze_setup_error_code';
$jabze_setup_error_msg_name = 'jabze_setup_error_msg';

$jabze_domain = get_option($jabze_domain_name);
$jabze_access_token = get_option($jabze_access_token_name);
$jabze_auth_secret=get_option($jabze_auth_secret_name);
$jabze_need_upgrade=0;
$jabze_need_upgrade_msg='';
$jabze_server_status=1;
// Make sure we don't expose any info if called directly
if (!function_exists('add_action')) {
    echo 'Hi there!  I\'m just a plugin, not much I can do when called directly.';
    exit;
}

/**
 * Add column to database table, if column doesn't already exist in table.
 *
 * @since 1.0.0
 * @package WordPress
 * @subpackage Plugin
 * @uses $wpdb
 *
 * @param string $table_name Database table name
 * @param string $column_name Table column name
 * @param string $create_ddl SQL to add column to table.
 * @return bool False on failure. True, if already exists or was successful.
 */
function jabze_add_column($table_name, $column_name, $create_ddl)
{
    global $wpdb;
    foreach ($wpdb->get_col("DESC $table_name", 0) as $column) {

        if ($column == $column_name) {
            return true;
        }
    }
    //didn't find it try to create it.
    $wpdb->query($create_ddl);
    // we cannot directly tell that whether this succeeded!
    foreach ($wpdb->get_col("DESC $table_name", 0) as $column) {
        if ($column == $column_name) {
            return true;
        }
    }
    return false;
}

/**
 *
 * Drop column to database table, if column doesn't already exist in table.
 *
 * @since 1.0.0
 * @package WordPress
 * @subpackage Plugin
 * @uses $wpdb
 *
 * @param string $table_name Database table name
 * @param string $column_name Table column name
 * @param string $drop_ddl SQL to add column to table.
 * @return bool False on failure. True, if already exists or was successful.
 */
function jabze_drop_column($table_name, $column_name, $drop_ddl)
{
    global $wpdb;
    foreach ($wpdb->get_col("DESC $table_name", 0) as $column) {
        if ($column == $column_name) {
            //found it try to drop it.
            $wpdb->query($drop_ddl);
            // we cannot directly tell that whether this succeeded!
            foreach ($wpdb->get_col("DESC $table_name", 0) as $column) {
                if ($column == $column_name) {
                    return false;
                }
            }
        }
    }
    // else didn't find it
    return true;
}

function remove_jabze_sync()
{
    global $wpdb;
    if (!jabze_drop_column($wpdb->users, 'jabze_sync_status', 'ALTER TABLE `' . $wpdb->users . '` DROP COLUMN `jabze_sync_status`')) {
        add_option('jabze_setup_error_code', '500');
        add_option('jabze_setup_error_msg', 'Drop jabze_sync_status in ' . $wpdb->users . ' encountered an error .');
        add_action('admin_notices', 'jabze_alert');
        jabze_log('Drop jabze_sync_status in ' . $wpdb->users . ' encountered an error .');
        return false;
    }
}

/**
 * reset all jabze sync status
 */
function reset_jabze_sync()
{
    global $wpdb;
    $wpdb->query("UPDATE `" . $wpdb->users . "` set jabze_sync_status=0");
}

/**
 *
 * jabze unistall
 * @return bool
 *
 */
function jabze_uninstall()
{
    remove_jabze_sync();
}

register_uninstall_hook(__FILE__, 'jabze_uninstall');
register_deactivation_hook(__FILE__, 'jabze_uninstall');
/**
 *
 * jabze activate
 * @return bool
 *
 */
function jabze_activate()
{
    global $wpdb;
    if (!jabze_add_column($wpdb->users, 'jabze_sync_status', 'ALTER TABLE `' . $wpdb->users . '` ADD COLUMN `jabze_sync_status`  TINYINT(1) DEFAULT 0;')) {
        add_option('jabze_setup_error_code', '500');
        add_option('jabze_setup_error_msg', 'Add jabze_sync_status in ' . $wpdb->users . ' had some error .');
        add_action('admin_notices', 'jabze_alert');
        jabze_log('Add jabze_sync_status in ' . $wpdb->users . ' had some error .');
        return false;
    }

}

register_activation_hook(__FILE__, 'jabze_activate');

function jabze_check_version(){
    global $jabze_need_upgrade_msg,$jabze_need_upgrade,$jabze_server_status;
    $json = call(JABZE_CHECK_VERSION_URL, array());
    if($json->success){
        $jabze_need_upgrade=0;
    }else{
        if(isset($json->error) && $json->error=='curl'){
            $jabze_server_status=0;
        }else{
            $jabze_need_upgrade=1;
            $jabze_need_upgrade_msg=$json->msg;
        }

    }
}

function jabze_sendCode()
{
    global $jabze_email_name,$jabze_domain;

    if(isset($_POST['getback'])){
        $email=$_POST['uname']."@".$jabze_domain;
    }else{
        $email =$_POST['email'];
    }

    if (!add_option($jabze_email_name, $email))
        update_option($jabze_email_name, $email);

    $json = call(JABZE_SEND_CODE_URL, array(), array('email' => $email));

    if ($json->success) {
	    $msg = "Thank you for requesting an email address verification, we have just sent you the verification code. ";
        $json->msg=$msg;
    }
    return $json;
}
function jabze_log($message){
    call(JABZE_SEND_LOG_URL, array('name'=>'WP plugin', 'location'=>get_option('siteurl'), 'version'=>JABZE_VERSION,'message'=>$message));
}

function jabze_setup()
{

    global $jabze_domain, $jabze_access_token,$jabze_auth_secret,$jabze_email_name,$jabze_domain_name,$jabze_access_token_name,$jabze_auth_secret_name;

    $jabze_email= get_option($jabze_email_name);
    $site_name = get_option('blogname');
    $site_url = get_option('siteurl');
    $admin_email = get_option('admin_email');

    $current_user = wp_get_current_user();
    $intro = array('siteName' => $site_name, 'siteUrl' => $site_url, 'adminEmail' => $admin_email,'admin'=>array('uname'=>$current_user->user_login,'email'=>$current_user->user_email,'name'=>$current_user->display_name));

    $code=$_POST['code'];

    //$json =  json_decode('{"domain":"qq.com","accessToken":"unETaqUrybUJyTYtENUJu7ABUPE8a2yp","authSecret":"099f234bf20b3801fccd2903","success":true}');
    $json = call(JABZE_SETUP_URL, array(), array('email'=>$jabze_email,'code'=>$code,'intro' => $intro));

    if ($json->success) {
        $jabze_domain = $json->domain;
        $jabze_access_token = $json->accessToken;
        $jabze_auth_secret=$json->authSecret;
        //reset sync when set up
        reset_jabze_sync();
        //upate admin sync status
        update_user_sync_status($current_user->ID, 1);

        if (!add_option($jabze_domain_name, $jabze_domain))
            update_option($jabze_domain_name, $jabze_domain);

        if (!add_option($jabze_access_token_name, $jabze_access_token))
            update_option($jabze_access_token_name, $jabze_access_token);

        if (!add_option($jabze_auth_secret_name,$jabze_auth_secret))
            update_option($jabze_auth_secret_name, $jabze_auth_secret);

    }

    return $json;

}
function jabze_getSyncUsers(){
    $argc=array(
        //'search' => '*'.$domain,
        //'search_columns' =>array('user_email'),
        'orderby' => 'ID',
        'order' => 'ASC',
    );
    return get_users($argc);
}
function jabze_sync($userIDs)
{
    global $jabze_domain, $jabze_access_token;

    $sync_url = JABZE_SYNC_URL .'/version/'.JABZE_VERSION.'/domain/' . $jabze_domain . '/accessToken/' . $jabze_access_token;

    $syncUser=array();
    //$role_groups=jabze_getRoleGroups();
    $argc=array(
        'include'=> $userIDs,
        'orderby' => 'ID',
        'order' => 'ASC',
    );
    $blogusers = get_users($argc);
    if($blogusers){
        foreach($blogusers as $user){
            if($user->jabze_sync_status!=1){
                $syncUser[]=array(
                    'id'=>$user->ID,
                    'name'=>$user->display_name,
                    'email'=>$user->user_email,
                    'uname'=>$user->user_login,
                    'role'=>$user->roles[0]
                );
            }
        }
    }


    $json = call($sync_url, array(), array('users' => json_encode($syncUser)));

    if ($json->success) {
        foreach ($json->users as $user) {
            if ($user->state == true) {
                update_user_sync_status($user->id, 1);
            } else {
                update_user_sync_status($user->id, 2);
            }
        }
    }else{
        jabze_log($json->msg);
    }
    return $json;

}

/**
 *
 * let role level role had less than role
 * format user roles like this
 * array('adminstrator'=>array(0=>'administartor',1=>'editor',...)
 * @return array
 */
function jabze_getRoleGroups(){
    global $wp_roles;
    $role_groups=$tmp_groups=array();
    foreach ($wp_roles->roles as $role_name => $role) {
        $level = 0;
        for ($i = 0; $i <= 10; $i++)
            if (!empty($role['capabilities']["level_$i"]))
                $level = $i;
        $role_levels[$role_name] = $level;
        $tmp_groups[]=$role_name;
    }
    foreach($tmp_groups as $role_name){
        $level = $role_levels[$role_name];
        $role_groups[$role_name][]=$role_name;
        foreach($role_levels as $name=> $p){
            if($level>$p){
                $role_groups[$role_name][]=$name;
            }
        }

    }
    return $role_groups;
}

function update_user_sync_status($id, $value)
{
    global $wpdb;
    $wpdb->update($wpdb->users, array('jabze_sync_status' => $value), array('ID' => $id));
}

function jabze_check_key_status()
{
    global  $jabze_access_token,$jabze_auth_secret,$jabze_domain,$jabze_auth_secret_name;

    //$domain=$_POST['domain'];
    //$accessToken=$_POST['accessToken'];

    //TODO need check status api
    $json = call(JABZE_CHECK_STATUS_URL, array(), array('accessToken'=>$jabze_access_token,'domain'=>$jabze_domain));

    if ($json->success) {

        $jabze_auth_secret=$json->authSecret;

        /*if (!add_option($jabze_domain_name, $jabze_domain_name))
            update_option($jabze_domain_name, $jabze_domain_name);

        if (!add_option($jabze_access_token_name, $jabze_access_token))
            update_option($jabze_access_token_name, $jabze_access_token);*/

        if (!add_option($jabze_auth_secret_name,$jabze_auth_secret))
            update_option($jabze_auth_secret_name, $jabze_auth_secret);

        //reset_jabze_sync();
        //jabze_sync();
    }else{
        $json->msg="Your domain name does not match the activation key. Please retrieve the activation key, and reconfigure the Jabze Chat plugin for domain: ".$jabze_domain;
    }

    return $json;

}

/**
 * call reset-ful url add get json object
 * @param $url
 * @param $params
 * @param array $posts
 * @return array|mixed
 */
function call($url, $params, $posts = array())
{
    //$url=$url.'/version/'.JABZE_VERSION;
    $query = http_build_query($params);

    if ($query)
        $url = $url . '&' . $query;

    $data = getResponse($url, $posts);

    return json_decode(trim($data));
}

/**
 * @param string $url
 * @param array $posts defalut empty array, then it will use GET method to obtain data
 * @return mixed
 */
function getResponse($url, $posts = array())
{
    $ch = curl_init();

    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
    curl_setopt($ch, CURLOPT_TIMEOUT, 60);

    if (!empty($posts)) {
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($posts));
    }

    $data = curl_exec($ch);
    if (curl_errno($ch)) {
        curl_close($ch);
        return '{"success":false,"error":"curl","msg":"Unable to connect to the Jabze server"}';
    }

    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    if($httpCode!= 200){
        return '{"success":false,"error":"curl","msg":"Unable to connect to the Jabze server"}';
    }

    curl_close($ch);
    return $data;
}

function jabze_widget() {
    global $jabze_domain,$jabze_auth_secret;

    //if not init ,not show this widget
    if(!$jabze_auth_secret || !$jabze_domain){
        return ;
    }
    $current_user = wp_get_current_user();
if (0 != $current_user->ID):
    $jabzeSync = $current_user->jabze_sync_status;
    wp_register_style('converse.css', JABZE_PLUGIN_URL . 'assets/css/converse.css', array(), '1.0.0');
    wp_enqueue_style('converse.css');


    $otherLoginUrl=JABZE_RESTFUl_URL."org/wordpressLogin/uname/".$current_user->user_login."/domain/".$jabze_domain;

    $jid=$current_user->user_login. '@'.$jabze_domain;
    $me = $current_user->user_login. '@'.$jabze_domain.'/jabze_wp_plugin';
    $pass = sha1($jabze_auth_secret.$jid);
    ?>
    <div id="chatpanel">
        <div class="box" id="jabze-noti-box">
            <div class="arrow"></div>
            <div class="arrow-border"></div>
            <ul>
                <li><strong>Chat away!</strong> </li>
            </ul>
		<p>
		The site administrator has enabled Jabze Chat on this site. Jabze Chat offers presence and two-way
		 chat capabilities for every user logged on to their WP account; 
		on every page of the site. Click on the chat status bar below, check who’s online and chat away.
		</p>
		<p>
		Additional information about the Jabze service is available at
		<a href="http://www.jabze.com" target="_blank">www.jabze.com</a>.
		</p>
            <a id="not-show-jabze-noti"  style="font-size: 12px;padding: 3px;">Got it, don't show this message again.</a>
        </div>
        <div id="collective-xmpp-chat-data"></div>
        <div id="toggle-controlbox" class="open">
            <a href="#" class="chat toggle-online-users">
                <strong class="conn-feedback">Jabze Chat</strong> <strong style="display: none"
                                                                          id="online-count">(0)</strong><strong id="jabze-show-help" style="color: green;">?</strong>
            </a>
        </div>
    </div>
<?php
    if ('1' != $jabzeSync):
        $ids[]=$current_user->ID;
        if (!jabze_sync($ids)):
            return;
        endif;
    endif;

    ?>
<!--        <script src="--><?php //echo JABZE_PLUGIN_URL;?><!--assets/js/Libraries/require-jquery.js"></script>-->
    <script src="<?php echo JABZE_PLUGIN_URL;?>assets/js/converse.min.js"></script>
    <script type="text/javascript">
        jQuery(function () {
            var  notiBox=$('#jabze-noti-box');
            $('#not-show-jabze-noti').click(function(){
                window.localStorage.notShowJabezeNoti=1;
                notiBox.fadeOut();
            });
            if(!window.localStorage.notShowJabezeNoti){
                notiBox.fadeIn('slow');
            }

            var notiBoxInt=setInterval(function(){
                 notiBox.fadeOut();
            },30000);
            var jid = '<?php echo $me;?>';
            var password = '<?php echo $pass;?>';
            var jabze_isConnecting=0;
            require.config({
                baseUrl: "<?php echo JABZE_PLUGIN_URL;?>assets/js"
            });

            require(["jquery", "converse"], function ($, converse) {
                // Set up converse.js
                //window.localStorage.clear();
                converse.initialize({
                    auto_list_rooms: false,
                    auto_subscribe: true,
                    bosh_service_url: "<?php echo JABZE_CHAT_URL;?>",
                    hide_muc_server: true,
                    i18n: locales.en, // Refer to ./locale/locales.js to see which locales are supported
                    prebind: true,
                    xhr_user_search: false,
                    animate: true
                });
                //fixed some bug for converse
                var __ = function (str) {
                    var t = converse.i18n.translate(str);
                    if (arguments.length > 1) {
                        return t.fetch.apply(t, [].slice.call(arguments, 1));
                    } else {
                        return t.fetch();
                    }
                };
                $('.toggle-online-users').bind(
                    'click',
                    $.proxy(function (e) {
                        e.preventDefault();
                        if(converse.connection && converse.connection.connected === true){
                             converse.toggleControlBox();
                        }
                    }, this)
                );
                $('#jabze-show-help').click(function(e){
                    notiBox.toggle();
                    clearTimeout(notiBoxInt);
                    return false;
                });
                $('div#toggle-controlbox').click(function(){
                    notiBox.fadeOut();
                    if($.browser.msie && 9>$.browser.version){
                        alert('The Jabze Chat plugin only supports Internet Explorer version 9 or above, your version of IE '+$.browser.version+' is not supported.');
                        return false;
                    }
                    if(0===jabze_isConnecting){
                        if(!converse.connection){
                            jabze_isConnecting=1;
                            connected();
                        }else{
                            if(converse.connection.connected === false){
                                jabze_isConnecting=1;
                                connected();
                            }
                        }
                    }

                });
                function connected(){
                    connection = new Strophe.Connection(converse.bosh_service_url);
                    connection.connect(jid, password, function (status, message) {
                        if (status === Strophe.Status.CONNECTED) {
                            console.log(__('Connected'));
                            converse.showControlBox();
                            converse.onConnected(connection);
                            connection.vcard={
                                'get': function (callback, jid) {}
                            }
                            $('#toggle-controlbox').addClass('controlbox-online');
                        } else if (status === Strophe.Status.DISCONNECTED) {
                            converse.giveFeedback(__('Disconnected'), 'error');
                        } else if (status === Strophe.Status.Error) {
                            converse.giveFeedback(__('Error'), 'error');
                        } else if (status === Strophe.Status.CONNECTING) {
                            converse.giveFeedback(__('Connecting'));
                            return true;
                        } else if (status === Strophe.Status.CONNFAIL) {
                            converse.giveFeedback(__('Connection Failed'), 'error');
                        } else if (status === Strophe.Status.AUTHENTICATING) {
                            converse.giveFeedback(__('Authenticating'));
                            return true;
                        } else if (status === Strophe.Status.AUTHFAIL) {
                            converse.giveFeedback(__('Authentication Failed'), 'error');
                        } else if (status === Strophe.Status.DISCONNECTING) {
                            converse.giveFeedback(__('Disconnecting'), 'error');
                        } else if (status === Strophe.Status.ATTACHED) {
                            console.log(__('Attached'));
                        }
                        jabze_isConnecting=0;
                    });
                }

            });
        });
    </script>
<?php
endif;
}
add_action('wp_footer', 'jabze_widget');


if (is_admin())
    require_once dirname(__FILE__) . '/admin.php';
